/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef X86_UTIL_H__
#define X86_UTIL_H__

/*
 * The maximum number of tokens for any command including the command itself
 */
#define MAX_TOKENS  16

struct ListElem;
struct ListHead;

typedef struct ListElem ListElem;
typedef struct ListHead ListHead;

struct ListElem {
    ListElem* pNext;
    ListElem* pPrev;
};

struct ListHead {
    ListElem tElem;
    Sys* pSys;
    const char* pcFile;
};

typedef enum {
    MAPPING_R_FIXED,
    MAPPING_W_FIXED,
    MAPPING_W_FIXED_MAY_CREATE,
    MAPPING_UIO
} MAPPING;

extern void InitListHead(ListHead* pListHead,Sys* pSys,const char* pcFile);
extern void InsertListElem(ListHead* pListHead,ListElem* pElem);
extern ListElem* CheckElems(
        ListHead* pListHead,
        bool (*rCheckElem)(ListHead* pListHead,ListElem* pListElem,void* pxArg),
        void* pxArg
        );
extern void FreeList(ListHead* pListHead);
extern void PrettyPrint(Sys* pSys,uint32_t qCount,uint8_t* pbBuf);
extern bool GetBaseNumber(
        Sys* pSys,
        const char* pcFile,
        int iLine,
        const char* pcVariable,
        char* pcToken,
        int iBase,
        uint32_t* pqVal
        );
extern int Tokenize(
        Sys* pSys,
        const char* pcFile,
        int iLine,
        char* pcBuf,
        const char* pcSep,
        int iMaxTokens,
        char* ppcTokens[]
        );
extern bool MapFile(
        Sys* pSys,
        const char* pcContext,
        const char* pcFile,
        MAPPING eMapping,
        uint64_t gOffset,
        uint32_t* pqRequiredSize,
        void** ppxMap
        );
extern bool UnmapFile(Sys* pSys,uint32_t qSize,void* pxMap);

#endif  /* X86_UTIL_H__ */
